module TweekCatalogue
  module Concerns
    module I18nSupport
      extend ActiveSupport::Concern

      included do
        class_attribute :supported_locales, :i18n_keys, :i18n_relations
      end

      module ClassMethods

        # Set boundary for supported locales.
        #
        # You can set which set of locales should be supported by +i18n_key+ and
        def i18n_keys_for(*locales)
          self.supported_locales = locales.map(&:to_sym)
        end

        def i18n_many(attr, opts = {})
          add_i18n_relationship!(attr)
          generate_i18n_many(attr, opts)
        end

        def i18n_one(attr, opts = {})
          add_i18n_relationship!(attr)
          generate_i18n_one(attr, opts)
        end

        def i18n_validates_presence_of(*fields)
          fields.each do |field|
            supported_locales.each do |locale|
              other_locales = supported_locales - [locale]
              other_fields = other_locales.map { |l| "#{field}_#{l}" }
              class_eval <<-CODE, __FILE__, __LINE__
                validates_presence_of :"#{field}_#{locale}", unless: -> { [#{other_fields.join('.present?, ')}].any? }
              CODE
            end
          end
        end

        # Create key with I18n support.
        #
        # Actually this field will create one ore more fields based on the
        # +i18n_keys_for+ locale list. Also additionally an accessor which will
        # available the attribute value based on the +I18n.locale+ settings.
        def i18n_key(attr_name, opts = {})
          add_i18n_key!(attr_name)
          generate_i18n_helper_methods! attr_name, opts
        end

        def i18n_fields
          @i18n_fields ||= (i18n_keys + i18n_relations).uniq
        end

        private

        def add_i18n_key! attr_name
          self.i18n_keys ||= []
          self.i18n_keys += [attr_name]
        end

        def add_i18n_relationship! attr_name
          self.i18n_relations ||= []
          self.i18n_relations += [attr_name]
        end

        def generate_i18n_one(attr_name, opts)
          supported_locales.each do |locale|
            one :"#{locale}_#{attr_name}", opts

            class_eval <<-CODE, __FILE__, __LINE__
              def #{attr_name}
                self.send(:"\#{I18n.locale}_#{attr_name}")
              end
            CODE
          end
        end

        def generate_i18n_many(attr_name, opts)
          supported_locales.each do |locale|
            many :"#{locale}_#{attr_name}", opts

            class_eval <<-CODE, __FILE__, __LINE__
              def #{attr_name}
                self.send(:"\#{I18n.locale}_#{attr_name}")
              end
            CODE
          end
        end

        def generate_i18n_helper_methods!(attr_name, opts)
          self.supported_locales ||= ['en']

          supported_locales.each do |locale|

            # Set mongomapper key
            key :"#{attr_name}_#{locale.to_s}"

            # Generate attr_name_for suffix which accepts locale parameter
            class_eval <<-CODE, __FILE__, __LINE__
              def #{attr_name}_for(locale)
                self.send :"#{attr_name}_\#{locale}"
              end
            CODE
          end
        end
      end
    end
  end
end
