module TweekCatalogue
  class Video

    include MongoMapper::Document

    include ::TweekCatalogue::Concerns::I18nSupport
    include ::TweekCatalogue::Concerns::ObjectFactories
    include ::TweekCatalogue::Concerns::HasManyRelationship

    # Support i18n fields with en and de locales
    i18n_keys_for :en, :de

    # Movie id references
    key :unique_tmdb_id, String
    key :tmdb_id, String
    key :imdb_id, String
    key :freebase_id, String
    key :freebase_mid, String
    key :tvdb_id, String
    key :tvrage_id, String

    # Movie financial, length and popularity related data
    key :budget, Float
    key :revenue, Float
    key :runtime, Float

    # Movie status related fields
    key :adult, Boolean
    key :status, String
    key :production, Boolean

    # Resource related path references
    key :backdrop_path, String
    key :poster_path, String

    key :original_title, String
    # Movie localized data attributes
    i18n_key :title, String
    i18n_key :overview, String
    i18n_key :tagline, String

    key :homepage, String
    key :first_air_at, Date
    key :last_air_at, Date

    key :episode_number
    key :season_number

    # Movie related objects
    it_has_many :production_companies
    it_has_many :production_countries
    it_has_many :spoken_languages
    it_has_many :genres
    it_has_many :keywords
    it_has_many :networks, class_name: 'TweekCatalogue::Network'

    it_has_many :creators,    class_name: 'TweekCatalogue::Participant'
    it_has_many :guest_stars, class_name: 'TweekCatalogue::Participant'
    it_has_many :crew,        class_name: 'TweekCatalogue::Participant'
    it_has_many :cast,        class_name: 'TweekCatalogue::Participant'

    # Embedded objects
    many :trailers, class_name: 'TweekCatalogue::Trailer'
    many :alternative_titles, class_name: 'TweekCatalogue::AlternativeTitle'
    many :releases, class_name: 'TweekCatalogue::Release'

    many :backdrops, class_name: 'TweekCatalogue::Image'
    many :stills,    class_name: 'TweekCatalogue::Image'
    many :posters,   class_name: 'TweekCatalogue::Image'

    key :serie_tmdb_id, String   # belongs_to :serie

    belongs_to :video_collection, class_name: 'TweekCatalogue::VideoCollection'

    timestamps!

    scope :recently_released, -> { sort(:first_air_at.desc) }

    before_destroy do
      Participant.where(:video_id => self.id).destroy_all
    end

    before_save do
      self.unique_tmdb_id = "#{self.class.to_s.split('::').last.underscore}_#{tmdb_id}"
    end

    #has_many seasons
    def seasons; [] end

    #has_many seasons
    def episodes; [] end

    # belongs_to :serie
    def serie; nil end

    # belongs_to :season
    def season; nil end
  end
end
